@@ -34,6 +34,7 @@
 #include <gnutls_state.h>
 #include <gnutls_datum.h>
 #include <extras/randomart.h>
+#include <read-file.h>
 
 /**
  * gnutls_random_art:
@@ -93,7 +94,7 @@ int gnutls_random_art (gnutls_random_art_t type,
 void
 gnutls_dh_set_prime_bits (gnutls_session_t session, unsigned int bits)
 {
-  if (bits < 512) _gnutls_audit_log(session, "Note that the security level of the Diffie-Hellman key exchange has been lowered to %u bits and this may allow decryption of the session data\n", bits);
+  if (bits <= 512) _gnutls_audit_log(session, "Note that the security level of the Diffie-Hellman key exchange has been lowered to %u bits and this may allow decryption of the session data\n", bits);
   session->internals.dh_prime_bits = bits;
 }
 
@@ -504,7 +505,7 @@ gnutls_certificate_get_ours (gnutls_session_t session)
   CHECK_AUTH (GNUTLS_CRD_CERTIFICATE, NULL);
 
   cred = (gnutls_certificate_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_CERTIFICATE, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_CERTIFICATE, NULL);
   if (cred == NULL || cred->certs == NULL)
     {
       gnutls_assert ();
@@ -551,6 +552,33 @@ gnutls_certificate_get_peers (gnutls_session_t
   return info->raw_certificate_list;
 }
 
+/**
+ * gnutls_certificate_get_peers_subkey_id:
+ * @session: is a gnutls session
+ * @id: will contain the ID
+ *
+ * Get the peer's subkey ID when OpenPGP certificates are
+ * used. The returned @id should be treated as constant.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ **/
+int gnutls_certificate_get_peers_subkey_id(gnutls_session_t session,
+                                           gnutls_datum_t *id)
+{
+  cert_auth_info_t info;
+
+  CHECK_AUTH (GNUTLS_CRD_CERTIFICATE, GNUTLS_E_INVALID_REQUEST);
+
+  info = _gnutls_get_auth_info (session);
+  if (info == NULL)
+    return GNUTLS_E_INVALID_REQUEST;
+    
+  id->data = info->subkey_id;
+  id->size = GNUTLS_OPENPGP_KEYID_SIZE;
+
+  return 0;
+}
 
 /**
  * gnutls_certificate_client_get_request_status:
@@ -565,7 +593,7 @@ gnutls_certificate_get_peers (gnutls_session_t
 int
 gnutls_certificate_client_get_request_status (gnutls_session_t session)
 {
-  return session->key->crt_requested;
+  return session->key.crt_requested;
 }
 
 /**
@@ -736,3 +764,65 @@ gnutls_anon_set_params_function (gnutls_anon_server_credentials_t res,
 {
   res->params_func = func;
 }
+
+/**
+ * gnutls_load_file:
+ * @filename: the name of the file to load
+ * @data: Where the file will be stored
+ *
+ * This function will load a file into a datum. The data are
+ * zero terminated but the terminating null is not included in length.
+ * The returned data are allocated using gnutls_malloc().
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ *
+ * Since 3.1.0
+ **/
+int gnutls_load_file(const char* filename, gnutls_datum_t * data)
+{
+size_t len;
+
+  data->data = (void*)read_binary_file(filename, &len);
+  if (data->data == NULL)
+    return GNUTLS_E_FILE_ERROR;
+  
+  if (malloc != gnutls_malloc)
+    {
+      void* tmp = gnutls_malloc(len);
+      
+      memcpy(tmp, data->data, len);
+      free(data->data);
+      data->data = tmp;
+    }
+  
+  data->size = len;
+  
+  return 0;
+}
+
+/**
+ * gnutls_url_is_supported:
+ * @url: A PKCS 11 url
+ *
+ * Check whether url is supported.  Depending on the system libraries
+ * GnuTLS may support pkcs11 or tpmkey URLs.
+ *
+ * Returns: return non-zero if the given URL is supported, and zero if
+ * it is not known.
+ *
+ * Since: 3.1.0
+ **/
+int
+gnutls_url_is_supported (const char* url)
+{
+#ifdef ENABLE_PKCS11
+  if (strstr(url, "pkcs11:") != NULL)
+    return 1;
+#endif
+#ifdef HAVE_TROUSERS
+  if (strstr(url, "tpmkey:") != NULL)
+    return 1;
+#endif
+  return 0;
+}
