@@ -26,6 +26,7 @@
 #include <gnutls/pkcs12.h>
 #include <gnutls/pkcs11.h>
 #include <gnutls/abstract.h>
+#include <gnutls/crypto.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -968,6 +969,9 @@ cmd_parser (int argc, char **argv)
 
   memset (&cinfo, 0, sizeof (cinfo));
   
+  if (HAVE_OPT(VERBOSE))
+    cinfo.verbose = 1;
+  
   if (HAVE_OPT(LOAD_PRIVKEY))
     cinfo.privkey = OPT_ARG(LOAD_PRIVKEY);
 
